@@ -5,6 +5,9 @@
 //! Key of removed entries are reused for new insertions.
 //! Underlying data is stored in a vector, keys are just indexes of that vector.
 //! The main trick is keeping in-place linked list of freed indexes for reuse.
+//!
+//! Serde is supported. If you need pre-computed length at serialization time
+//! (for example, for bincode), use serde_ser_len feature.
 
 
 #[cfg(test)]
@@ -27,6 +30,15 @@ enum Entry<V> {
     Occupied(V)
 }
 
+impl<V> Entry<V> {
+    fn is_empty(&self) -> bool {
+        match self {
+            &Entry::Empty(_) => true,
+            _ => false,
+        }
+    }
+}
+
 /// A map that chooses small integer keys for you.
 /// You store something into this map and then access it by ID returned by it.
 /// For small V entries are expected to take 16 bytes.
@@ -237,6 +249,50 @@ impl<V> CompactMap<V> {
     pub fn len_slow(&self) -> usize {
         self.iter().count()
     }
+    
+    /// Trims the `CompactMap` of any excess capacity.
+    ///
+    /// Rescans the whole map to reindex empty slots. O(n).
+    ///
+    /// The collection may reserve more space to avoid frequent reallocations.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use compactmap::CompactMap;
+    /// let mut map: CompactMap<&str> = CompactMap::with_capacity(10);
+    /// assert_eq!(map.capacity(), 10);
+    /// map.shrink_to_fit();
+    /// assert_eq!(map.capacity(), 0);
+    /// map.insert("qwe");
+    /// map.insert("345");
+    /// map.insert("555");
+    /// map.remove(1);
+    /// map.shrink_to_fit();
+    /// assert_eq!(map.capacity(), 2);
+    /// ```
+    pub fn shrink_to_fit(&mut self) {
+        // strip off trailing `Empty`s
+        if let Some(idx) = self.data.iter().rposition(Entry::is_empty) {
+            self.data.truncate(idx + 1);
+        } else {
+            self.data.clear();
+        };
+
+        self.data.shrink_to_fit();
+        self.reindex();
+    }
+
+    
+    fn reindex(&mut self) {
+        self.free_head = usize::MAX;
+        for i in 0..self.data.len() {
+            if let Entry::Empty(ref mut head) = self.data[i] {
+                *head = self.free_head;
+                self.free_head = i;
+            }
+        }
+    }
 }
 
 impl<V> Default for CompactMap<V> {
@@ -395,9 +451,6 @@ impl<V: fmt::Debug> fmt::Debug for CompactMap<V> {
     }
 }
 
-
-//TODO: compaction?
-
 macro_rules! generate_iterator {
     ($self_:ident, mut) => {
         generate_iterator!($self_ ; & mut Entry::Occupied(ref mut x), x);
@@ -487,4 +540,89 @@ impl<V> IntoIterator for CompactMap<V> {
     }
 }
 
+#[cfg(feature = "serde")]
+mod serdizer {
+    extern crate serde;
+    
+    use super::CompactMap;
+    use super::Entry;
+    
+    use std::usize;
+    use self::serde::ser::SerializeMap;
+    
+    impl<V:serde::Serialize> serde::Serialize for CompactMap<V> {
+        fn serialize<S : serde::Serializer>(&self, s:S) -> Result<S::Ok, S::Error> {
+            #[cfg(feature = "serde_ser_len")]
+            let len = Some(self.len_slow());
+            #[cfg(not(feature = "serde_ser_len"))]
+            let len = None;
+            
+            let mut map = s.serialize_map(len)?;
+            for (k, v) in self {
+                map.serialize_entry(&k, v)?;
+            }
+            map.end()
+        }
+    }
+    
+    // Deserializer based on https://serde.rs/deserialize-map.html
+    
+    use std::fmt;
+    use std::marker::PhantomData;
+    
+    use self::serde::de::{Deserialize, Deserializer, Visitor, MapAccess};
+    
+    struct MyMapVisitor<V> {
+        marker: PhantomData<fn() -> CompactMap<V>>
+    }
+    
+    impl<V> MyMapVisitor<V> {
+        fn new() -> Self {
+            MyMapVisitor {
+                marker: PhantomData
+            }
+        }
+    }
+    
+    impl<'de, V> Visitor<'de> for MyMapVisitor<V>
+        where V: Deserialize<'de>
+    {
+        type Value = CompactMap<V>;
+    
+        // Format a message stating what data this Visitor expects to receive.
+        fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+            formatter.write_str("a map with small nonnegative integer keys")
+        }
 
+        fn visit_map<M>(self, mut access: M) -> Result<Self::Value, M::Error>
+            where M: MapAccess<'de>
+        {
+            let mut map = CompactMap::with_capacity(access.size_hint().unwrap_or(0));
+    
+            while let Some((key, value)) = access.next_entry()? {
+            
+                // because of Vec::resize_default is unstable
+                while map.data.len() <= key {
+                    map.data.push(Entry::Empty(usize::MAX));
+                }
+                map.data[key] = Entry::Occupied(value);
+            }
+            map.reindex();
+    
+            Ok(map)
+        }
+    }
+    
+    // This is the trait that informs Serde how to deserialize MyMap.
+    impl<'de, V> Deserialize<'de> for CompactMap<V>
+        where V: Deserialize<'de>
+    {
+        fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+            where D: Deserializer<'de>
+        {
+            // Instantiate our Visitor and ask the Deserializer to drive
+            // it over the input data, resulting in an instance of MyMap.
+            deserializer.deserialize_map(MyMapVisitor::new())
+        }
+    }
+}
